/** @file
 * @brief Logger class implementation. This class implements the Logging facility.
 *
 *
 *
 * @copyright Intra2net AG
 * @license GPLv2
*/


#include "logger.h"

#include <iostream>
#include <syslog.h>
#include <sstream>

#include <boost/foreach.hpp>

#define VERSION     0
#define REVISION    1
#define RELEASE     0

namespace po = boost::program_options;

typedef boost::shared_ptr<boost::program_options::options_description> Options_descriptionPtr;

using namespace std;

/**
 * Default Constructor
 */
Logger::Logger()
    : Loglevel(0)
    , Syslog(false)
    , ExternalWarningLevel(0)
{
    set_log_facility(Loglevel,Syslog,ExternalWarningLog,ExternalWarningLevel);
}


/**
 * Default Destructor
 */
Logger::~Logger()
{
}


/**
 * Decides if Logging through syslog if enabled or through std.
 * @param msg The message to log.
 */
void Logger::log_notice(const string& msg, int level) const
{
    if ( Syslog )
        syslog(LOG_NOTICE,msg.c_str());
    else
        cout << msg << endl;;
}


/**
 * Decides if Logging through syslog if enabled or through std.
 * @param msg The message to log.
 */
void Logger::log_warning(const string& msg, int level) const
{
    if ( Syslog )
        syslog(LOG_WARNING,msg.c_str());
    else
        cout << msg << endl;

    if ( (level <= ExternalWarningLevel) && (!ExternalWarningLog.empty()) )
    {
        string external = ExternalWarningLog;
        external.append(" ");
        external.append("\"");
        external.append(msg);
        external.append("\"");
        if ( system(external.c_str()) != 0 )
            print_error_external_logging(external);
    }
}


/**
 * Decides if Logging through syslog if enabled or through std.
 * @param msg The message to log.
 */
void Logger::log_error(const string& msg, int level) const
{
    if ( Syslog )
        syslog(LOG_ERR,msg.c_str());
    else
        cerr << msg << endl;
}


/**
 * Setter for member Loglevel.
 * @param _loglevel Value to set Loglevel to.
 */
void Logger::set_loglevel(const int _loglevel)
{
    Loglevel = _loglevel;
    cout << "Loglevel set" << endl;
}


/**
 * Getter for member Loglevel.
 * @return Loglevel.
 */
int Logger::get_loglevel() const
{
    return Loglevel;
}


/**
 * Setter for member Syslog.
 * @param _syslog Wether to log through syslog or not.
 */
void Logger::set_syslog(const bool _syslog)
{
    Syslog = _syslog;
}


/**
 * Getter for member Syslog.
 * @return True if logging through syslog is enabled, false otherwise.
 */
bool Logger::get_syslog() const
{
    return Syslog;
}


/**
 * Initialize the logging facility.
 */
void Logger::set_log_facility(const int _loglevel, const bool _syslog, const string& _external_error_log, const int _external_error_level)
{
    Loglevel = _loglevel;
    Syslog = _syslog;
    ExternalWarningLog = _external_error_log;
    ExternalWarningLevel = _external_error_level;


    if ( Syslog )
        openlog("bpdyndnsd",LOG_PID,LOG_DAEMON);
}


/**
 * Prints out the usage to the command line.
 */
void Logger::print_usage(const Options_descriptionPtr opt_desc) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Usage: bpdyndnsd [Command line options]" << "\n" << endl;
        msg << *opt_desc << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the programm name and the given version string on stdout.
 * @param version Version string.
 */
void Logger::print_version() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Bullet proof dynamic dns daemon.\nbpdyndnsd " << VERSION << "." << REVISION << "." << RELEASE << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the successful parsing of the command line options.
 */
void Logger::print_cmd_parsed() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        log_notice("Command line options successfully parsed.",level);
    }
}


void Logger::print_conf_files_parsed() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        log_notice("Config files successfully parsed.",level);
    }
}


/**
 * Prints out the successful parsing of the config files.
 * @param config_path The specified config path.
 */
void Logger::print_conf_loaded(const string& config_path) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Config files successfully loaded in: " << config_path << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the unsuccessful parsing of the config files.
 * @param config_path The specified config path.
 */
void Logger::print_conf_not_loaded(const string& config_path) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Config files couldn't be loaded in: " << config_path << endl;
        log_error(msg.str(),level);
    }
}


/**
 * A file could not be opened for reading.
 * @param filename The filename.
 */
void Logger::print_error_opening_r(const string& filename) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error opening file for reading: " << filename << endl;
        log_error(msg.str(),level);
    }
}


/**
 * A file could not be opened for writing.
 * @param filename The filename.
 */
void Logger::print_error_opening_rw(const string& filename) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error opening file for writing: " << filename << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Desctructor of specified class was called.
 * @param _class Name of the class.
 */
void Logger::print_destructor_call(const string& _class) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Destructor call: " << _class << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Constructor of specified class was called.
 * @param _class Name of the class.
 */
void Logger::print_constructor_call(const string& _class) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Constructor call: " << _class << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Update method for specified service was called.
 * @param service The service for which is the update running.
 */
void Logger::print_update_service(const string& service) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Running update for service: " << service << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * An unknown option on the command line was detected.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_cmd_option(const string& unknown_option) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Unknown option on command line detected: " << unknown_option << endl;
        log_error(msg.str(),level);
    }
}


/**
 * An unknown protocol was specified.
 * @param protocol The unknown protocol.
 */
void Logger::print_unknown_protocol(const string& protocol) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Unknown protocol defined: " << protocol << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Loading a service config file.
 * @param filename The service config file.
 */
void Logger::print_load_service_conf(const string& filename) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Loading service config file: " << filename << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Loading the main config file.
 * @param filename The main config file.
 */
void Logger::print_load_main_conf(const string& filename) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Loading main config file: " << filename << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * There is an unknown option in a service config file.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_service_conf_option(const string& service_conf_file, const string& unknown_option) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Unknown option in service config file detected: " << service_conf_file << " Unknown option: " << unknown_option << endl;
        log_error(msg.str(),level);
    }
}


/**
 * There is an unknown option in the main config file.
 * @param unknown_option The unknown option.
 */
void Logger::print_unknown_main_conf_option(const string& unknown_option) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Unknown option in main config file detected: " << unknown_option << endl;
        log_error(msg.str(),level);
    }
}


/**
 * The defined config path doesn't exist or is not a directory.
 * @param config_path The defined config path.
 */
void Logger::print_error_config_path(const string& config_path) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Config path doesn't exists or is not a diretory: " << config_path << endl;
        log_error(msg.str(),level);
    }
}


/**
 * There is a missing command line option to initialize a service object.
 */
void Logger::print_missing_cmd_service_option() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_error("Missing option to initialize service. Protocol, host, login and password must be specified.",level);
    }
}


/**
 * Missing option in service config file.
 * @param service_conf_file Service config file
 */
void Logger::print_missing_service_conf_option(const string& service_conf_file) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Missing option in service config file " << service_conf_file << " to initialize service. Protocol, host, login and password must be specified." << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Process running as daemon.
 * @param pid The pid of the daemon.
 */
void Logger::print_runnig_as_daemon(const int pid) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Runnig as daemon: " << pid << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the daemon mode.
 * @param daemon_mode The daemon mode.
 */
void Logger::print_daemon_mode(const bool daemon_mode) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        string mode = "disabled";
        if (daemon_mode == true)
            mode = "enabled";
        ostringstream msg;
        msg << "Daemon mode is " << mode << "." << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * There was an error while trying to fork.
 */
void Logger::print_error_fork() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_error("Error while trying to fork.",level);
    }
}


/**
 * A pid in the pidfile was found.
 * @param pid The pid found in the pidfile.
 */
void Logger::print_pid_found(const int pid) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Pidfile found: " << pid << ". Checking if process is still runnig..." << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Another process is already running.
 * @param pid The pid of the other process.
 */
void Logger::print_process_already_running(const int pid) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Another bpdyndnsd process with PID " << pid << " is already running!" << endl;
        log_error(msg.str(),level);
    }
}


/**
 * SIGTERM caught.
 */
void Logger::print_caught_sigterm() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Caught SIGTERM. Exiting...",level);
    }
}


/**
 * SIGUSR1 caught.
 */
void Logger::print_caught_siguser1() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Caught SIGUSR1. Switching to offline mode...",level);
    }
}


/**
 * SIGHUP caught.
 */
void Logger::print_caught_sighup() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        log_notice("Caught SIGHUP. Reloading config and switching to online mode...",level);
    }
}


/**
 * Error while setting signal handler.
 */
void Logger::print_error_setting_signal(const string& signal) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while setting signal handler for: " << signal << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Error while setting signal handler.
 */
void Logger::print_init_log_facility() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Initialized logging facility." << " Loglevel: " << Loglevel << " Syslog: " << Syslog << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Be verbose. Currently we are in offline mode.
 */
void Logger::print_offline_mode() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Offline mode...",level);
    }
}


/**
 * Objects successfully serialized.
 */
void Logger::print_serialized_objects_success() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        log_notice("Serialized objects successfully.",level);
    }
}


/**
 * Objects successfully de-serialized.
 */
void Logger::print_deserialized_objects_success() const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        log_notice("De-serialized objects successfully.",level);
    }
}


/**
 * Prints out the content of a service object.
 * @param message Message to be added on output first.
 * @param protocol Service's protocol.
 * @param hostname Service's hostname.
 * @param login Service's login.
 * @param password Service's password.
 * @param actual_ip Service's actual_ip.
 * @param lastupdated Service's lastupdated.
 */
void Logger::print_service_object(const string& message, const string& protocol, const string& hostname, const string& login, const string& password, const int update_interval, const int max_updates_within_interval, const int dns_cache_ttl , const string& actual_ip, list<int> lastupdated) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << message << endl;
        msg << "\t" << "Protocol:         " << protocol << endl;
        msg << "\t" << "Hostname:         " << hostname << endl;
        msg << "\t" << "Login:            " << login << endl;
        msg << "\t" << "Password:         " << password << endl;
        msg << "\t" << "Update Interval:  " << update_interval << endl;
        msg << "\t" << "Max Updates:      " << max_updates_within_interval << endl;
        msg << "\t" << "DNS Cache TTL:    " << dns_cache_ttl << endl;
        msg << "\t" << "Actual_IP:        " << actual_ip << endl;
        BOOST_FOREACH( int update_time, lastupdated)
        {
            msg << "\t" << "Lastupdated:      " << update_time << endl;
        }
        log_notice(msg.str(),level);
    }
}


/**
 * Caught exception while serialize.
 * @param exception Exception message.
 */
void Logger::print_exception_serialize(const string& exception) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while trying to serialize Serviceholder object: " << exception << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Caught exception while de-serialize.
 * @param exception Exception message.
 */
void Logger::print_exception_deserialize(const string& exception) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while trying to de-serialize Serviceholder object: " << exception << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Child couldn't be killed by parent.
 * @param pid Pid of the child.
 */
void Logger::print_error_kill_child(const int pid) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Could not kill child process with PID: " << pid << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Child was killed by parent because of error.
 * @param pid The pid (child) killed.
 */
void Logger::print_child_killed(const int pid) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Killed child process with PID: " << pid << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * There is no object file.
 * @param object_file The object file.
 */
void Logger::print_no_object_file(const string& object_file) const
{
    int level = 1;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "There is no object file: " << object_file << ". Continue without recovering state from old services!" << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Prints out the given hostname
 * @param hostname Hostname as string.
 */
void Logger::print_hostname(const string& hostname) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Detected following hostname of localhost: " << hostname << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the detected own ipv4 address
 * @param ip_addr String representation of the detected ip.
 */
void Logger::print_own_ipv4(const string& ip_addr_v4, const string& hostname) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Detected following IPv4-Address of host: " << hostname << " : " << ip_addr_v4 << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Prints out the detected own ipv5 address
 * @param ip_addr String representation of the detected ip.
 */
void Logger::print_own_ipv6(const string& ip_addr_v6, const string& hostname) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Detected following IPv6-Address of host: " << hostname << " : " << ip_addr_v6 << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Exception while trying to resolve hostname to ip.
 * @param exception The exception caught.
 * @param hostname The hostname.
 */
void Logger::print_error_hostname_to_ip(const string& exception, const string& hostname) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Could not resolve the hostname: " << hostname << " to an IP-Address: " << exception << endl;
        log_error(msg.str(),level);
    }
}


/**
 * The update of the given service was successful.
 * @param service The service.
 */
void Logger::print_update_service_successful(const string& service) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Updated service successful: " << service << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * No ip could be determined through webcheck
 */
void Logger::print_webcheck_no_ip() const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        log_warning("IP-Address of this host could not be determined through any configured webcheck url.",level);
    }
}


/**
 * Connection problem while trying to get ip through webcheck url
 * @param curl_err_buff Curl error message
 * @param url the url
 */
void Logger::print_webcheck_url_connection_problem(const char * curl_err_buff, const string& url) const
{
    int level = 1;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "There was a problem while trying to connect to following URL: " << url << " CURL error: " << curl_err_buff << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Prints out curl error.
 * @param curl_err_buff Curl error message.
 * @param url URL
 */
void Logger::print_webcheck_error(const char * curl_err_buff, const string& url) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "There was an error while trying to connect to following URL: " << url << " CURL error: " << curl_err_buff << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Prints out the received data through curl.
 * @param curl_data Data string
 */
void Logger::print_received_curl_data(const string& curl_data) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Received CURL data: " << curl_data << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * IP was foudn through regex
 * @param ip The IP found.
 */
void Logger::print_regex_found_ip(const string& ip) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Found IP-Address via regex: " << ip << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * No IP was found through regex.
 * @param data The data string which should contain a valid IP.s
 */
void Logger::print_regex_ip_not_found(const string& data) const
{
    int level = 1;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Could not extract an IP-Address via regex from following data:\n" << data << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Detected multiple occurrences of the same option.
 * @param message Error message.
 */
void Logger::print_multiple_cmd_option(const string& message) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "The same option is only allowed once: " << message << endl;
        log_error(msg.str(),level);
    }
}


/**
 * An update would exceed the update interval. Prints out a warning message.
 * @param current_time Current time.
 * @param old_time Time of update #MaxUpdatesWithinInterval ago.
 * @param MaxUpdatesWithinInterval Number of allowed updates in one update interval.
 * @param service The service which exceeds update interval.
 */
void Logger::print_update_not_allowed(const int current_time, const int old_time, const int MaxUpdatesWithinInterval, const string& service) const
{
    int level = 1;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Update not allowed for service: " << service << ". Too many updates within max update interval. Current time: " << current_time << ". Update time before " << MaxUpdatesWithinInterval << " updates: " << old_time << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Failure while running update for service.
 * @param service Services' name.
 */
void Logger::print_update_service_failure(const string& service) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Could not update service: " << service << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Starting shutdown
 */
void Logger::print_starting_shutdown() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Shutting down ...",level);
    }
}


/**
 * Shutdown complete
 */
void Logger::print_shutdown_succeeded() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Shutting down complete ...",level);
    }
}


/**
 * Shutdown parent succeeded
 */
void Logger::print_shutdown_parent_succeeded() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Shutting down parent process completed ...",level);
    }
}


/**
 * Starting shutdown parent
 */
void Logger::print_starting_shutdown_parent() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_notice("Shutting down parent process ...",level);
    }
}


/**
 * DNS cache record timeout
 * @param hostname Hostname
 * @param lastupdated Lastupdated
 * @param dns_cache_ttl DNS cache TTL
 * @param current_time Current time
 */
void Logger::print_recheck_dns_entry(const string& hostname, const int lastupdated, const int dns_cache_ttl, const int current_time) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "DNS cache record for host <" << hostname << "> expired or host will be updated for the first time: Lastupdated: " << lastupdated << " DNS cache ttl: " << dns_cache_ttl << " Current time: " << current_time << " Checking current DNS cache status." << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Missing proxy option on command line.
 */
void Logger::print_missing_cmd_proxy_option() const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        log_error("Missing option to initialize proxy. http_proxy and http_proxy_port must be specified.",level);
    }
}


/**
 * Multiple option in service config file.
 * @param service_conf_file Service config file
 * @param message Multiple option text
 */
void Logger::print_multiple_service_conf_option(const string& service_conf_file, const string& message) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Multiple occurrences of the same option in service config file detected: " << service_conf_file << " " << message << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Multiple option in main config file.
 * @param service_conf_file Service config file
 * @param message Multiple option text
 */
void Logger::print_multiple_main_conf_option(const string& main_conf_file, const string& message) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Multiple occurrences of the same option in main config file detected: " << main_conf_file << " " << message << endl;
        log_error(msg.str(),level);
    }
}


/**
 * Missing proxy option in main config file.
 * @param main_conf_filename The concerning config file.
 */
void Logger::print_missing_conf_proxy_option(const string& main_conf_filename) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Missing option to initialize proxy in main config file: " << main_conf_filename << " http_proxy and http_proxy_port must be specified." << endl;
        log_error(msg.str(),level);
    }
}


/**
 * There is no domain part in the given hostname
 * @param hostname The hostname with no domain part in it.
 */
void Logger::print_no_domain_part(const string& hostname) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "There is no domain part in the given hostname: " << hostname << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * An curl error occured.
 * @param url The url requested by the curl operation
 * @param curl_err_code The resulting curl error code
 */
void Logger::print_curl_error(const string& url, const int curl_err_code) const
{
    string curl_err = "";

    if ( curl_err_code == 3 )
        curl_err = "CURLE_URL_MALFORMAT";
    else if ( curl_err_code == 6 )
        curl_err = "CURLE_COULDNT_RESOLVE_HOST";
    else if ( curl_err_code == 7 )
        curl_err = "CURLE_COULDNT_CONNECT";
    else
        curl_err = "UNKNOWN";

    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Curl error while requesting following url: " << url << " Curl error code: " << curl_err_code << " "<< curl_err << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * An curl error occured.
 * @param url The url requested by the curl operation
 * @param curl_err_code The resulting curl error code
 * @param curl_err_buff The curl error buffer
 */
void Logger::print_curl_error(const string& url, const int curl_err_code, const char * curl_err_buff) const
{
    string curl_err = "";

    if ( curl_err_code == 3 )
        curl_err = "CURLE_URL_MALFORMAT";
    else if ( curl_err_code == 6 )
        curl_err = "CURLE_COULDNT_RESOLVE_HOST";
    else if ( curl_err_code == 7 )
        curl_err = "CURLE_COULDNT_CONNECT";
    else
        curl_err = "UNKNOWN";

    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Curl error while requesting following url: " << url << " Curl error code: " << curl_err_code << " "<< curl_err << " " << curl_err_buff << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Prints out the data received by curl operation
 * @param CurlWritedataBuff 
 */
void Logger::print_curl_data(const string& curl_writedata_buff) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Data received by curl: " << curl_writedata_buff << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Not authorized to perform requested update operation
 * @param service The requested service.
 * @param username Username
 * @param password Password
 */
void Logger::print_service_not_authorized(const string& service, const string& username, const string& password) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Not authorized to perform update operation on service: " << service << " Please check username and password: " << username << ":" << password << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Prints out the http status code
 * @param url Url
 * @param output HTTP status code
 */
void Logger::print_http_status_code(const string& url, const long http_code) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Requested URL: " << url << " Received HTTP status code: " << http_code << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Generic failure while trying to update service
 * @param url The requested URL
 * @param curl_data The received curl_data from the server
 */
void Logger::print_update_failure(const string& url, const string& curl_data) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Problem while trying to updating service. Requested URL: " << url << " Error Code from Server: " << curl_data << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Generic failure while trying to update service
 * @param url The requested URL
 * @param http_status_code The received http status code
 */
void Logger::print_update_failure(const string& url, const long http_status_code) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Problem while trying to updating service. Requested URL: " << url << " Error Code from Server: " << http_status_code << endl;
        log_warning(msg.str(),level);
    }
}

/**
 * Hostname is invalid, contains no or only one domain part.
 * @param hostname The full qualified host name. 
 */
void Logger::print_invalid_hostname(const string& hostname) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "The configured hostname: " << hostname << " is invalid. Please add the corresponding domain part." << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * An IP in a private range was detected
 * @param ip The private IP
 */
void Logger::print_ip_is_local(const string& ip) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "The detected IP is within a private IP range: " << ip << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Regex is matching in string
 * @param regex The regex pattern
 * @param matching_string The string
 */
void Logger::print_regex_match(const string& regex, const string& matching_string) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Regex: " << regex << " is matching in: " << matching_string << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Regex is not matching
 * @param regex Regex
 * @param not_matching_string String
 */
void Logger::print_no_regex_match(const string& regex, const string& not_matching_string) const
{
    int level = 1;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Regex: " << regex << " is not matching in: " << not_matching_string << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Could not parse gnudip initial reply.
 * @param curl_data The data received from gnudip server which should contain salt, time and sign.
 */
void Logger::print_could_not_parse_received_data(const string& curl_data) const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        ostringstream msg;
        msg << "Could not parse salt, time and sign from initial gnudip server reply: " << curl_data << endl;
        log_warning(msg.str(),level);
    }
}


/**
 * Gnudip salt, time and sign could not be got from map
 */
void Logger::print_could_not_get_initial_gnudip_data() const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        log_warning("Could not get salt, time and sign from map.",level);
    }
}



/**
 * Gnudip protocol requires explicit declaration of a servername.
 */
void Logger::print_gnudip_requires_servername() const
{
    int level = 0;
    if ( (level <= Loglevel) || ((level <= ExternalWarningLevel) && (!ExternalWarningLog.empty())) )
    {
        log_warning("Gnudip requires explicit definition of servername via config!",level);
    }
}


/**
 * An exception occured while computing the md5 sum.
 * @param what The exception occured.
 */
void Logger::print_exception_md5_sum(const string& what) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "An exception occured while computing a md5 sum: " << what << endl;
        log_error(msg.str(),level);
    }
}


/**
 * An network exception occured.
 * @param what The exception occured.
 */
void Logger::print_network_error(const string& what) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "An netowrk exception occured: " << what << endl;
        log_error(msg.str(),level);
    }
}


/**
 * An undefined protocol error occured.
 * @param protocol The protocol
 * @param error The error
 */
void Logger::print_undefined_protocol_error(const string& protocol, const string& error) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "An undefined protocol error occured. Protocol: " << protocol << " Error: " << error << endl;
        log_error(msg.str(),level);
    }
}

/**
 * Error while trying to log through external program.
 * @param external_prog The external program called.
 */
void Logger::print_error_external_logging(const string& external_prog) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while trying to log through external program: " << external_prog << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Error while parsing config file.
 * @param error Error occured.
 * @param config_file Config file.
 */
void Logger::print_error_parsing_config_file(const string& filename, const string& error) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while parsing config file: " << filename << " Error: " << error << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Error while parsing cmd option
 * @param error Error
 */
void Logger::print_error_parsing_cmd(const string& error) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Error while parsing cmd options: " << error << endl;
        log_error(msg.str(),level);
    }
}


/**
 * The webcheck interval was exceeded, so webcheck not allowed.
 * @param last_webcheck Time of last webcheck.
 * @param webcheck_interval Webcheck interval time.
 * @param current_time Current system time.
 */
void Logger::print_webcheck_exceed_interval( const int last_webcheck, const int webcheck_interval, const int current_time ) const
{
    int level = 0;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Exceeding webcheck interval: LastWebcheck " << last_webcheck << " + WebcheckInterval(sec) " << webcheck_interval << " >= CurrentTime " << current_time << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Checking if hosts needs update.
 * @param hostname Hostname
 * @param current_time Current time
 * @param lastupdated Last updated
 */
void Logger::print_check_service_update(const string& hostname, const int current_time, const int lastupdated) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Checking if host: " << hostname << " needs update. Current time: " << current_time << " Last updated: " << lastupdated << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Cached DNS entry
 * @param hostname Hostname
 * @param ip_dns_recheck DNS recheck IP
 * @param ip_last_update IP set in last update
 * @param ip_host Hosts' IP
 */
void Logger::print_cached_dns_entry(const string& hostname, const string& ip_dns_recheck, const string& ip_last_update, const string& ip_host) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Cached DNS record for host <" << hostname << "> : " << ip_dns_recheck << " Last updated IP: " << ip_last_update << " Hosts IP: " << ip_host << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Updating service for the first time.
 * @param hostname Hostname
 * @param ip_dns_recheck Cached DNS entry
 * @param ip_host Hosts IP
 */
void Logger::print_update_service_firttime(const string& hostname, const string& ip_dns_recheck, const string& ip_host) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Updating service for the first time. Hostname: " << hostname << " DNS-Record: " << ip_dns_recheck << " Hosts IP: " << ip_host << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * Updating service
 * @param hostname Hostname
 * @param ip_dns_recheck Cached DNS entry
 * @param ip_last_update IP set in last update
 * @param ip_host Hosts IP
 * @param lastupdated Lastupdated
 */
void Logger::print_update_service(const string& hostname, const string& ip_dns_recheck, const string& ip_last_update, const string& ip_host, const int lastupdated) const{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Updating service. Hostname: " << hostname << " DNS-Record: " << ip_dns_recheck << " IP set in last update: " << ip_last_update << " Lastupdated: " << lastupdated << " Hosts IP: " << ip_host << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * TTL expired
 * @param hostname Hostname
 * @param ip_dns_recheck Cached DNS entry
 * @param ip_last_update IP set in last update
 * @param ip_host Hosts IP
 * @param lastupdated Last updated
 * @param dns_cache_ttl DNS cache ttl
 * @param current_time Current time
 */
void Logger::print_update_service_ttl_expired(const string& hostname, const string& ip_dns_recheck, const string& ip_last_update, const string& ip_host, const int lastupdated, const int dns_cache_ttl, const int current_time) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "TTL for service expired and still pointing to old IP. Hostname: " << hostname << " DNS-Record: " << ip_dns_recheck << " IP set in last update: " << ip_last_update << " Lastupdated: " << lastupdated << "DNS Cache TTL: " << dns_cache_ttl << " Current Time: " << current_time << " Hosts IP: " << ip_host << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * TTL expired
 * @param hostname Hostname
 * @param ip_dns_recheck Cached DNS entry
 * @param ip_last_update IP set in last update
 * @param ip_host Hosts IP
 * @param lastupdated Last updated
 * @param dns_cache_ttl DNS cache ttl
 * @param current_time Current time
 */
void Logger::print_update_service_ttl_not_expired(const string& hostname, const string& ip_dns_recheck, const string& ip_last_update, const string& ip_host, const int lastupdated, const int dns_cache_ttl, const int current_time) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "Waiting for DNS cache TTL to expire. Hostname: " << hostname << " DNS-Record: " << ip_dns_recheck << " IP set in last update: " << ip_last_update << " Lastupdated: " << lastupdated << "DNS Cache TTL: " << dns_cache_ttl << " Current Time: " << current_time << " Hosts IP: " << ip_host << endl;
        log_notice(msg.str(),level);
    }
}


/**
 * No update needed
 * @param hostname Hostname
 * @param ip_dns_recheck Cached DNS entry
 * @param ip_last_update IP set in last update
 * @param ip_host Hosts IP
 * @param lastupdated Last updated
 */
void Logger::print_no_update_needed(const string& hostname, const string& ip_dns_recheck, const string& ip_last_update, const string& ip_host, const int lastupdated) const
{
    int level = 1;
    if ( level <= Loglevel )
    {
        ostringstream msg;
        msg << "No update needed for host: " << hostname << " Cached DNS record: " << ip_dns_recheck << " IP set in last update: " << ip_last_update << " Hosts IP: " << ip_host << " Last updated: " << lastupdated << endl;
        log_notice(msg.str(),level);
    }
}
